/*
 * $Id: ChatAuth.java,v 1.6 2006/10/13 08:53:01 smeyer Exp $
 */
package databay.chatserver;

import org.apache.log4j.Logger;

import java.util.Vector;
import java.util.Hashtable;
import java.util.StringTokenizer;
import java.util.Enumeration;

import java.net.Socket;
import java.net.InetAddress;

public class ChatAuth
{
		private Logger logger = Logger.getLogger(this.getClass().getName());

		private static ChatAuth instance = null;
		private static final int LIMIT = 2000;

		private int counter = 0;
		private boolean auth_enabled = false;

		private Hashtable hash = new Hashtable();
		private Vector vector = new Vector();
		private Vector allowed_ips = new Vector();
		
		/** 
		 * Singleton
		 * Privater Konstruktur. Benutze getInstance()
		 **/
		private ChatAuth()
		{
		}
		/** 
		 * Erzeugt eine Instanz, wenn noch keine existiert
		 **/
		public static ChatAuth getInstance()
			throws Exception
		{
			if(instance == null)
			{
				instance = new ChatAuth();
			}
			else
			{
				throw new Exception("ChatAuth already created");
			}
			return instance;
		}

		public void setAllowedIps(String ips)
		{
			StringTokenizer stk = new StringTokenizer(ips,",");
			try
			{
				while(stk.hasMoreTokens())
				{
					InetAddress ip = InetAddress.getByName(stk.nextToken());
					allowed_ips.addElement(ip);
					logger.debug("Added allowed Ip: " + ip);
				}
			}
			catch(Exception e)
			{
				logger.error("CONFIG: " + e.toString());
			}
		}

		public void enableAuth()
		{
			auth_enabled = true;
		}
		public void disableAuth()
		{
			auth_enabled = false;
		}
		public boolean isEnabled()
		{
			return auth_enabled;
		}

		public synchronized void addUser(String uname, InetAddress ip)
		{
			if(!isEnabled())
			{
				return;
			}
			if(counter == LIMIT)
			{
				deleteFirst();
			}
			hash.put(uname,ip);
			logger.debug("Added user: " + uname + " Ip: " + ip);
			vector.addElement(uname);
			++counter;

			return;
		}

		public synchronized void deleteUser(String uname)
		{
			if(!isEnabled())
			{
				return;
			}
			if(counter != 0)
			{
				hash.remove(uname);
				vector.removeElementAt(vector.indexOf(uname,0));
				logger.debug("Deleted user: " + uname);
				--counter;
			}
		}
		public boolean checkWriteAccess(Socket s)
			throws NoWriteAccessException
		{
			if(!isEnabled())
			{
				return true;
			}

			for(Enumeration el = allowed_ips.elements();el.hasMoreElements();)
			{
				if(el.nextElement().equals(s.getInetAddress()))
				{
					return true;
				}
			}
			throw new NoWriteAccessException("No write access for " + s.getInetAddress());
		}
			

		public boolean checkReadAccess(String uname,Socket s)
			throws NoReadAccessException
		{
			if(!isEnabled())
			{
				logger.debug("Check Access disabled: " + s.getInetAddress());
				return true;
			}
			try
			{
				logger.debug("Check read access: Write access granted: " + s.getInetAddress());
				checkWriteAccess(s);
				return true;
			}
			catch(NoWriteAccessException e)
			{
				// DO NOTHING
			}
            logger.debug("Nickname: " + uname);
			InetAddress ip = (InetAddress) hash.get(uname);
			logger.debug("Stored IP: " + ip.toString());
          
            
			if(ip == null || !ip.equals(s.getInetAddress()))
			{
                logger.debug("Check read access: FAILED for " + s.getInetAddress() + " (" + ip + ")");
				throw new NoReadAccessException("No read access for " + s.getInetAddress());
			}
			return true;
		}

		// PRIVATE
		private void deleteFirst()
		{
			hash.remove(vector.elementAt(0));
			vector.removeElementAt(0);
			logger.debug("Entry deleted");
			--counter;
		}
}

class NoWriteAccessException extends Exception
{
		/**
     * 
     */
    private static final long serialVersionUID = 1L;

        public NoWriteAccessException(String msg)
		{
			super(msg);
		}
}
class NoReadAccessException extends Exception
{
		/**
     * 
     */
    private static final long serialVersionUID = 1L;

        public NoReadAccessException(String msg)
		{
			super(msg);
		}
}
