/*
 * $Id: ChatRoomBuffers.java,v 1.5 2005/06/30 08:10:23 krichter Exp $
 */
package databay.chatserver;

import java.util.Enumeration;
import java.util.Hashtable;

/**
 * 
 * Zentraler Speicher fuer alle Chatrooms
 *  
 */
public class ChatRoomBuffers
{
    private Hashtable myChatRooms;
    private int       defaultLen;

    /**
     * erzeugt den zentralen Chatroom-Speicher und setzt die Standard
     * Bufferlaenge auf bufferlength
     *  
     */
    public ChatRoomBuffers(int bufferlength)
    {
        defaultLen = bufferlength;
        myChatRooms = new Hashtable();
    }

    /**
     * gibt zurueck ob ein Chatroom mit dem gegebenen Namen existiert
     *  
     */
    public boolean existsChatRoom(String name)
    {
        if (myChatRooms.containsKey(name))
        {
            return true;
        } else
        {
            return false;
        }
    }

    public boolean deleteChatRoom(String name)
    {
        if (!myChatRooms.containsKey(name))
        {
            return false;
        }
        myChatRooms.remove(name);
        return true;
    }

    /**
     * fuegt einen neuen Chatroom in den Buffer ein
     *  
     */
    public ChatRoom addChatRoom(String name)
    {
        if (name.length()==0)
            return null;
        
        if (myChatRooms.isEmpty())
        {
            ChatRoom cr = new ChatRoom(name, defaultLen);
            myChatRooms.put(name, cr);
            return cr;
        }
        if (myChatRooms.containsKey(name))
        {
            return null;
        } 
        else
        {
            ChatRoom cr = new ChatRoom(name, defaultLen);
            myChatRooms.put(name, cr);
            return cr;
        }
    }

    /**
     * gibt den Speicher des uebergebenen Chatrooms zurueck
     * 
     * @param String
     *            name des Chatrooms
     * @return Ringbuffer die Daten des Chatrooms
     */
    public ChatRoom getChatRoom(String name)
    {
        return (ChatRoom) myChatRooms.get(name);
    }

    /**
     * gibt alle Chatroom-Namen zurueck
     * 
     * @return Enumeration Namen
     */
    public Enumeration getAllChatRooms()
    {
        return myChatRooms.keys();
    }

    /**
     * @param chatRoomName
     */
    public boolean clearChatRoom(String chatRoomName)
    {
        if (!myChatRooms.containsKey(chatRoomName))
        {
            return false;
        }
        ChatRoom r = (ChatRoom) myChatRooms.get(chatRoomName);
        r.getBuffer().clear();
        return true;
    }
}