/*
 * $Id: ChatServer.java,v 1.15 2006/10/13 08:53:01 smeyer Exp $
 */
package databay.chatserver;

import java.net.ServerSocket;
import java.net.Socket;
import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Logger;

/**
 * dies ist die Hauptklasse, die den ChatServer startet.
 */
public class ChatServer
{
		private Logger logger = null;
        public static String version= new String("3.7.3");
		private ChatSpeicher cs =  null;
		private ChatAuth ca = null;	

		/**
		 * der ChatServer liest die INI Datei ein und baut Sockets fuer eingehende Verbindungen auf.
		 * jede Verbindung ist ein eigener Thread in dem eine ServerCon laeuft
		 * @see databay.chatserver.ServerCon
		 *
		 */
		public ChatServer(String[] args)
		{
			ServerSocket server = null;
			Socket s = null;
			ServerCon doIt;
			Thread t;
			
            initLogger();
	
			this.cs = new ChatSpeicher(this);

			if (args.length > 0)
			{
				if ( ! cs.readIniFile(args[0])) {
				    logger.error("Cannot parse INI-File.");
                    System.exit(1);
				}

			}
			else 
			{
				logger.error("Error: No INI-File given.");
				System.exit(0);
			}

			// hole auth objejekt
			try
			{
				ca = ChatAuth.getInstance();
				if(cs.authenticationEnabled())
				{
					ca.enableAuth();
					ca.setAllowedIps(cs.getAllowedIps());
				}
			}
			catch(Exception e)
			{
				logger.error(e.toString());
				System.exit(1);
			}
	
			int count = 0;  // Vielviel Connections 
	
			System.out.println("Chat-Server start...");
			logger.info("Start threading ...");

			if(cs.sslEnabled())
			{
				ChatServerSocket csocket_ssl = new ChatServerSocket(this,true);
				csocket_ssl.setIp(cs.getServerIP());
				csocket_ssl.setPort(cs.getSSLPort());
				csocket_ssl.setKeyStore(cs.getSSLKeyStoreFile());
				csocket_ssl.setKeyStorePassword(cs.getSSLKeyStorePassword());
				csocket_ssl.setKeyPassword(cs.getSSLKeyPassword());

				Thread t1 = new Thread(csocket_ssl);
				t1.start();
			}

			ChatServerSocket csocket = new ChatServerSocket(this,false);
			csocket.setIp(cs.getServerIP());
			csocket.setPort(cs.getPort());

			Thread t2 = new Thread(csocket);
			t2.start();
		}

		public ChatSpeicher getChatSpeicher()
		{
			return this.cs;
		}
		public ChatAuth getChatAuth()
		{
			return this.ca;
		}

		private void initLogger() {
      
            logger = Logger.getLogger(ChatServer.class);
            //logger.setAdditivity(false);
            BasicConfigurator.configure();
            
            return;
        }
        
        
        
		public static void main (String[] args) 
		{
			ChatServer server = new ChatServer(args);
		}
}

