/*
 * $Id: ChatServerSocket.java,v 1.6 2006/10/13 08:53:01 smeyer Exp $
 */
package databay.chatserver;

import java.io.FileInputStream;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.security.KeyStore;
import javax.net.ServerSocketFactory;
import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLServerSocket;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;
import org.apache.log4j.Logger;

public class ChatServerSocket extends Thread
{
		private Logger logger = Logger.getLogger(this.getClass().getName());
		private ChatServer cserver = null;

		private boolean sslEnabled = false;
		private int port = 0;
		private InetAddress ip = null;
		private String keyStore;
		private String keyStorePassword;
		private String keyPassword;

		private static int counter = 0;
		private static final int MAX_CONNECTIONS = 10;

		public ChatServerSocket(ChatServer cserver,boolean sslEnabled)
		{
			this.cserver = cserver;
			this.sslEnabled = sslEnabled;
		}

		public boolean sslEnabled()
		{
			return this.sslEnabled;
		}
		public void setPort(int port)
		{
			this.port = port;

			return;
		}
		public int getPort()
		{
			return this.port;
		}

		public void setIp(InetAddress ip)
		{
			this.ip = ip;

			return;
		}
		public InetAddress getIp()
		{
			return this.ip;
		}
		public void setKeyStore(String ks)
		{
			this.keyStore = ks;
		}
		public String getKeyStore()
		{
			logger.debug("getKeyStore: " + this.keyStore);
			return this.keyStore;
		}
		public void setKeyStorePassword(String kpw)
		{
			this.keyStorePassword = kpw;
		}
		public String getKeyStorePassword()
		{
			return this.keyStorePassword;
		}
		public void setKeyPassword(String kpw)
		{
			this.keyPassword = kpw;
		}
		public String getKeyPassword()
		{
			return this.keyPassword;
		}
			
		private ServerSocket getServer()
			throws Exception
		{
			KeyStore ks = null;
			KeyManagerFactory kmf = null;
			SSLContext sslc;
			ServerSocketFactory ssf = null;

			try
			{
				ks = KeyStore.getInstance("PKCS12");
				ks.load(new FileInputStream(this.getKeyStore()),this.getKeyStorePassword().toCharArray());
			
				kmf = KeyManagerFactory.getInstance("SUNX509");
				kmf.init(ks,this.getKeyPassword().toCharArray());


				// ALLEN ZERTIFIKATEN TRAUEN
				TrustManager[] trustAllCerts = new TrustManager[] 
					{ 
						new X509TrustManager()
						{
								public java.security.cert.X509Certificate[] getAcceptedIssuers()
								{
									return null;
								}
									
								public void checkClientTrusted(
									java.security.cert.X509Certificate[] certs,
									String authType)
								{
								}
										
								public void checkServerTrusted(
									java.security.cert.X509Certificate[] certs,
									String authType)
								{
								}
						}
					};


				sslc = SSLContext.getInstance("SSLv3");
				sslc.init(kmf.getKeyManagers(),trustAllCerts,new java.security.SecureRandom());
		  
				ssf = sslc.getServerSocketFactory();

				return (SSLServerSocket) ssf.createServerSocket(this.getPort(),MAX_CONNECTIONS,this.getIp());
			}
			catch(Exception e)
			{
				logger.fatal(e.getMessage());
				throw new Exception(e.getMessage());
			}
		}
	

		// DIE run Methode bindet an den SSL und nicht-SSL Port
		// und startet dann die einzelnen Threads fuer die Verbindungen
		public void run()
		{
			ServerSocket server = null;
			Socket socket = null;

			// SSL BIND
			try
			{
				if(sslEnabled())
				{
					server = this.getServer();

					// Allen Server Zertifikaten trauen
					/*
					TrustManager[] trustAllCerts = new TrustManager[] 
						{ 
							new X509TrustManager()
							{
									public java.security.cert.X509Certificate[] getAcceptedIssuers()
									{
										return null;
									}
									
									public void checkClientTrusted(
										java.security.cert.X509Certificate[] certs,
										String authType)
									{
									}
										
									public void checkServerTrusted(
										java.security.cert.X509Certificate[] certs,
										String authType)
									{
									}
							}
						};
					*/
					logger.info("(SSL) Bound to: Ip: " + getIp() + " Port: " + getPort());
				}
				else
				{
					if(getIp() != null)
					{
						server = new ServerSocket(getPort(),0,getIp());
					}
					else
					{
						server = new ServerSocket(getPort());
					}
					logger.info("Bound to: Ip: " + getIp() + " Port: " + getPort());
				}					
				
				// Starte Threads fuer eingehende Verbindungen
				while(true)
				{
					socket = server.accept();

					// Erh?he statischen Z?hler
					logger.debug("Port: " + socket.getPort());
					logger.debug("Local Port: " + socket.getLocalPort());

					if(sslEnabled())
					{
						logger.info("New SSL connection:" + ++counter);
					}
					else
					{
						logger.info("New connection:" + ++counter);
					}

					ServerCon sv = new ServerCon(socket,cserver.getChatSpeicher(),cserver.getChatAuth());

					Thread t1 = new Thread(sv);
					t1.start();
				}
			}
			catch (Exception e) 
			{
                if ( server!=null)
                {
    				try 
    				{ 
    					server.close(); 
    				} 
    				catch (Exception e2) 
    				{
    					logger.fatal("ServerClose "+e2);
    				}
                }
    			logger.fatal("ServerMain " +e);
    			System.exit(1);
			}
		}
}					
					
					
		
