/*
 * $Id: ChatSpeicher.java,v 1.14 2005/08/09 11:26:23 smeyer Exp $
 */
package databay.chatserver;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.net.InetAddress;
import java.util.Enumeration;
import java.util.StringTokenizer;
import java.util.Vector;

import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.PatternLayout;
import org.apache.log4j.RollingFileAppender;



/** 
 * Zentrales Speicherobjekt das neben den ChatRoomBuffers auch die Header- und Footer Zeilen enthaelt.
 */
public class ChatSpeicher {

		private Logger logger = Logger.getLogger(ChatSpeicher.class);
        private Vector messages = new Vector();
        
		private static int count = 0;
		private int linesPerChatRoom = 100;
		private ChatServer server =null;
		private ChatRoomBuffers myBuffers;
		private Vector headerContents;
		private Vector footerContents;
		private String iniFileName;
		private String headerFileName;
		private String footerFileName;
		private String adminPassword;
		private String moderatorPassword;
		private int auth_enabled;
		private int ssl_enabled = 0;
		private boolean startEmpty;
		private String sslKeyStoreFile;
		private String sslKeyStorePassword;
		private String sslKeyPassword;

		private int port = 6678;
		private int ssl_port = 6679;
		private String ipAddr=null;
		private InetAddress serverIP = null;
		private String allowedIps;
        private String defaultStyleSheet="http://www.databay.de/chat/default.css";
        private String defaultCharacterEncoding="utf-8";
        private long defaultExpirationTime=3600;
        
		/**
		 * der ChatSpeicher bekommt den Vector der header- und footer Zeilen sowie eine Referenz auf den ChatServer uebergeben.
		 */
		public ChatSpeicher(ChatServer server)
		{
			myBuffers = new ChatRoomBuffers(linesPerChatRoom);

			this.server=server;
		}

		/**
		 * gibt das Admin Passwort zurueck
		 *
		 */
		public String getAdminPassword()
		{
			return adminPassword;
		}

		/**
		 * gibt das Moderator Passwort zurueck
		 *
		 */
		public String getModeratorPassword()
		{
			return moderatorPassword;
		}
    
		/**
		 * gibt zurueck ob Authentifizierung in server.ini aktiviert wurde
		 */
		public boolean authenticationEnabled()
		{
			return auth_enabled == 1 ? true : false;
		}

		/**
		 * gibt zurueck ob SSL in server.ini aktiviert wurde
		 */
		public boolean sslEnabled()
		{
			return ssl_enabled == 1 ? true : false;
		}
		/**
		 * gibt die Ip(s) zurueck von denen schreibzugriff erfolgen kann
		 */
		public String getAllowedIps()
		{
			return allowedIps;
		}


    
		/**
		 * gibt zurueck ob das StartEmpty Flag gesetzt ist
		 * Dieses Flag dient dazu, die Ausgabe der letzten BufferSize/2 Zeilen beim Betreten zu unterdruecken.
		 */
		public boolean getStartEmpty()
		{
			return startEmpty;	
		}

		/**
		 * gibt zurueck welchen Port der Server verwenden soll
		 * Default: 6678
		 */
		public int getPort()
		{
			return port;
		}

		/**
		 * gibt zurueck welcher SSL-Port der Server verwenden soll
		 * Default: 6679
		 */
		public int getSSLPort()
		{
			return ssl_port;
		}

		/**
		 * gibt die ServerIP zurueck falls angegeben
		 * @return InetAddress serverIP
		 */
		public InetAddress getServerIP()
		{
			return serverIP;
		}

		/**
		 * gibt das Admin Passwort aus
		 *  @return String AdminPassword
		 */
		public String getAdminPasswd()
		{
			return adminPassword;
		}

		/**
		 * gibt das Moderator Passwort zurueck
		 * @return String ModeratorPasswort
		 */
		public String getModeratorPasswd()
		{
			return moderatorPassword;
		}

		/**
		 * gibt die Header aus der Header Datei aus
		 */
		public void printHeaders(PrintWriter pw,ChatRoom cr)
		{
			for (Enumeration e= headerContents.elements(); e.hasMoreElements();)
			{
                String s = (String) e.nextElement();
                if (cr.getStylesheetURL()!=null)
                {
                    s = s.replaceAll("#STYLE#",cr.getStylesheetURL());
                }
                else if (getDefaultStyleSheet()!=null)
                {
                        s = s.replaceAll("#STYLE#",getDefaultStyleSheet());
                }
                
                if (cr.getCharacterEncoding()!=null)
                {
                    s = s.replaceAll("#CHARSET#",cr.getCharacterEncoding());
                }
                else if (getDefaultCharacterEncoding()!=null)
                {
                    s = s.replaceAll("#CHARSET#",getDefaultCharacterEncoding());
                }
				pw.println(s);
			}
		}

		/**
         * @return
         */
        public String getDefaultCharacterEncoding()
        {
            return defaultCharacterEncoding;
        }

        /**
		 * gibt die Footer Zeilen aus der Footer Datei aus
		 */
		public void printFooters(PrintWriter pw)
		{
			for (Enumeration e= footerContents.elements(); e.hasMoreElements();)
			{
				String s = (String) e.nextElement();
				pw.println(s);
			}
		}

		/** 
		 * @parameters String ChatRoomName
		 * @retuns boolean true falls ChatRoom existiert
		 */
		public boolean existsChatRoom(String chatRoomName)
		{
			return myBuffers.existsChatRoom(chatRoomName);
		}

		/**
		 * gibt alle ChatRoomNamen zurueck
		 */
		public Enumeration getAllRoomNames()
		{
			return myBuffers.getAllChatRooms();
		}

		/**
		 * erzeugt einen neuen ChatRoom 
		 * @param String chatRoomName
		 */
		public ChatRoom addChatRoom(String chatRoomName)
		{
            logger.debug("Add ChatRoom: \""+chatRoomName+"\"");
			ChatRoom cr = myBuffers.addChatRoom(chatRoomName);
            cr.setExpirationTime(getDefaultExpirationTime());
            
            return cr;
            
		}

		public void deleteChatRoom(String chatRoomName)
		{
			myBuffers.deleteChatRoom(chatRoomName);
		}

		public String getSSLKeyStoreFile()
		{
			return sslKeyStoreFile;
		}
		public String getSSLKeyStorePassword()
		{
			return sslKeyStorePassword;
		}
		public String getSSLKeyPassword()
		{
			return sslKeyPassword;
		}
    
		/**
		 * Methode zum Lesen des Chatspeichers.
		 * @param ConnectionData verbindungsdaten
		 * @param String nickname
		 * @param String chatRoomName
		 */
		public synchronized Vector readSpeicher(ConnectionData connectionData,String Nickname,String chatRoomName) 
		{
			Vector outputLines = new Vector();
			ChatRoom myChatRoom; 
            RingBuffer chatRoomBuffer;
			int myChatRoomCapacity; 
			int min,max;
			boolean firstTime=false;

			int pos = connectionData.getPosition();
			if (!myBuffers.existsChatRoom(chatRoomName))
			{
				myBuffers.addChatRoom(chatRoomName);
			}
            
			myChatRoom  = myBuffers.getChatRoom(chatRoomName);
            chatRoomBuffer = myChatRoom.getBuffer();
			myChatRoomCapacity = myChatRoom.getBuffer().getCapacity();

			min = chatRoomBuffer.getMinIndex(); 
			max = chatRoomBuffer.getMaxIndex();
	
			// Verbindung eben erst aufgebaut daher start Position : letzte 50 Zeilen
			if (pos==-2) 
			{
				if (startEmpty )
				{
					pos=max;	
				}	
				else 
				{
					if (max > myChatRoomCapacity/2)
						pos=(max-myChatRoomCapacity/2) % myChatRoomCapacity;
					else pos=-1;
				}
				firstTime=true;
				connectionData.setPosition(pos);
			}
			else 
			{
				firstTime=false;
			}

			int k = 0;
			if ((max>=0) && (pos!=max))
			{
				//		logger.debug("starting with i="+i+" maximum="+max);
				while (pos!= max) // (&& k++<10)
				{
					pos = (pos +1) % myChatRoomCapacity;
					LineEntry line = chatRoomBuffer.getLine(pos);

                    // suppress expired messages 
                    long now = System.currentTimeMillis()/1000;
                    if (line.getTimestamp() < now - myChatRoom.getExpirationTime())
                    {
                        logger.debug("Line expired: "+line.getTimestamp()+" < "+now +"-"+myChatRoom.getExpirationTime());
                        continue;
                    }
                        
                    
                    String toNick = line.getToNick();
					String fromNick = line.getFromNick();
					String message  = line.getMessage();

					if (toNick.equals("") || 
						toNick.equals(Nickname) || 
						fromNick.equals(Nickname))
					{
						if (! firstTime || ! message.startsWith("/"))
							outputLines.addElement(message);
					}
				}
	    
				connectionData.setPosition(max);
			}
			if (outputLines.size() >0) 
				logger.info("OUTPUT for "+Nickname+": "+outputLines);
			return outputLines;
		}
   
		/**
		 * methode zum einfuegen einer Zeile in den Chatroom.
		 * ist synchronized, damit der Speicher immer nur von einem User gleichzeitig
		 * modifiziert werden kann.
		 * @param String Nachtricht
		 * @param String an wen geht die Nachricht
		 * @param String von wem ist die Nachricht
		 * @param String Chatroom
		 */
		public synchronized void writeSpeicher(String message,String toNick,String fromNick,String chatRoom) 
		{
			RingBuffer myChatRoom;
			if (!myBuffers.existsChatRoom(chatRoom))
				myBuffers.addChatRoom(chatRoom);
			myBuffers.getChatRoom(chatRoom).addLine(new LineEntry(message,toNick,fromNick));
		}

		/**
		 * Methode zum Einlesen der Ini-Datei.
		 *
		 */    
		public boolean readIniFile(String iniFileName) 
		{
			String s, line;

            removeMessages();
            // Statische Informationen aus Definitionsfile einlesen
			try {
				FileReader fi = new FileReader(iniFileName);
				BufferedReader di = new BufferedReader(fi);
				while ((line = di.readLine()) != null)
				{
					StringTokenizer stk = new StringTokenizer(line, "\n \r \t=");
		
					if (stk.hasMoreTokens()== false){
						continue;
					}
		
					s = stk.nextToken();
		
					// Config File 
					if (s.equalsIgnoreCase("Port"))
					{
						port = Integer.parseInt(stk.nextToken());
                        addMessage("Config: Port = " + port);
					}
					else if(s.equalsIgnoreCase("SSLPort"))
					{
						ssl_port = Integer.parseInt(stk.nextToken());
                        addMessage("Config: SSLPort = " + ssl_port);
					}
					else if(s.equalsIgnoreCase("LogLevel") )
					{
					    Level level = Level.INFO;
						int settings_level = Integer.parseInt (stk.nextToken());
						switch(settings_level)
						{
							case 0:
								level = Level.OFF;
								break;

							case 1:
								level = Level.FATAL;
								break;

							case 2:
							    level = Level.ERROR;
								break;

							case 3:
							    level = Level.INFO;
								break;

							case 4:
							    level = Level.WARN;
								break;

							case 5:
							    level = Level.DEBUG;
								break;

							case 6:
							    level = Level.ALL;
								break;
						}
						try 
						{
							// SETZE den level des root loggers, alle logger erben hiervon.
							this.setRootLogLevel(level);
						}
						catch (Exception e)
						{

						}
					}
					else if (s.equalsIgnoreCase("StartEmpty"))
					{
						String startEmptyStr = stk.nextToken();
						if (startEmptyStr.equals("1"))
							startEmpty=true;
						else 
							startEmpty=false;
					}	
					else if(s.equalsIgnoreCase("IpAddress") )
					{
						ipAddr = stk.nextToken();
						serverIP = InetAddress.getByName(ipAddr);
						addMessage("Config: IPAddress = "+serverIP);
					}
					else if(s.equalsIgnoreCase("HeaderFileName") )
					{
						headerFileName = stk.nextToken();
						logger.info("Config: HeaderFileName = "+headerFileName);
					}
                    else if(s.equalsIgnoreCase("DefaultStylesheet") )
                    {
                        defaultStyleSheet = stk.nextToken();
                        addMessage("Config: DefaultStyleSheet = "+defaultStyleSheet);
                    }
                    else if(s.equalsIgnoreCase("DefaultCharset") )
                    {
                        defaultCharacterEncoding = stk.nextToken();
                        addMessage("Config: DefaultCharset = "+defaultCharacterEncoding);
                    }
					else if(s.equalsIgnoreCase("FooterFileName") )
					{
						footerFileName = stk.nextToken();
						addMessage("Config: FooterFileName = "+footerFileName);
					}
					else if (s.equalsIgnoreCase("AdminPassword") )
					{
						adminPassword = stk.nextToken();
						addMessage("Config: AdminPassword = "+adminPassword);
					}
					else if (s.equalsIgnoreCase("ModeratorPassword") )
					{
						moderatorPassword = stk.nextToken();
						addMessage("Config: ModeratorPassword = "+moderatorPassword);
					}
					else if (s.equalsIgnoreCase("Authentication"))
					{
						auth_enabled = Integer.parseInt(stk.nextToken());
						addMessage("Config: Authentication = " + auth_enabled);
					}
					else if(s.equalsIgnoreCase("SSL"))
					{
						ssl_enabled = Integer.parseInt(stk.nextToken());
						addMessage("Config: SSL = " + ssl_enabled);
					}
					else if (s.equalsIgnoreCase("ConnectionsFrom"))
					{
						allowedIps = stk.nextToken();
						addMessage("Config: ConnectionsFrom = "+allowedIps);
					}
					else if(s.equalsIgnoreCase("LogFile"))
					{
						String logfile = stk.nextToken();
						this.initLogger(logfile);
					}
					else if(s.equalsIgnoreCase("SSLKeyStoreFile") )
					{
						sslKeyStoreFile = stk.nextToken();
						addMessage("Config: SSLKeyStoreFile = " + sslKeyStoreFile);
					}
					else if(s.equalsIgnoreCase("SSLKeyStorePassword") )
					{
						sslKeyStorePassword = stk.nextToken();
						addMessage("Config: SSLKeyStorePassword = ******");
					}
					else if(s.equalsIgnoreCase("SSLKeyPassword") )
					{
						sslKeyPassword = stk.nextToken();
						addMessage("Config: SSLKeyPassword ******");
					}
                    else if(s.equalsIgnoreCase("DefaultExpirationTime") )
                    {
                        try 
                        {
                            defaultExpirationTime = Integer.parseInt(stk.nextToken());
                        }
                        catch (Exception ex)
                        {
                            defaultExpirationTime=3600;
                        }
                    }

				}
                showMessages();
				di.close();
				fi.close();
			}
			catch(IOException e)
			{

				if (e instanceof FileNotFoundException)
				{
					logger.warn("INI-Datei nicht gefunden!");
				}
				logger.fatal(e.toString());
                return false;
			}
            
            boolean result = readHTMLFiles();

            return result;
		}		

		/**
         * @return
         */
        public boolean readHTMLFiles()
        {
            if (! headerFileName.equals(""))
            {
                headerContents = catFileToVector(headerFileName);
            }
            if (! footerFileName.equals(""))
            {
                footerContents = catFileToVector(footerFileName);
            }

            return true;
        }

        /**
		 * Hilfsfunktion fuer das Umwandeln einer Datei in einen Vector der die Zeilen enthaelt.
		 */
		public  Vector catFileToVector(String fileName)
		{
	
			String line;
			Vector outputVector = new Vector();
	
			try 
			{
				FileReader fi = new FileReader(fileName);
				BufferedReader di = new BufferedReader(fi);
				while ((line = di.readLine()) != null)
				{
					outputVector.addElement(line);
				}
				di.close();
				fi.close();
			}
			catch(FileNotFoundException e)
			{
				logger.warn("Datei nicht gefunden: "+fileName);
			}
			catch(IOException e){
				logger.fatal(e.toString());
			}

			return outputVector;
    					
		}

		/**
		 * Routine fuer die Ausgabe eines Vector of String 
		 */
		public String catVector(Vector v)
		{
            String retval ="";
			for (Enumeration e= v.elements(); e.hasMoreElements();)
			{
				retval+=(String) e.nextElement();
			}
            return retval;
		}
        
        public ChatRoom getChatRoom(String name)
        {
            ChatRoom cr = myBuffers.getChatRoom(name);
            return cr;
        }
 
  
        public String getDefaultStyleSheet()
        {
            return defaultStyleSheet;
        }
        
        public void setDefaultStyleSheet(String defaultStyleSheet)
        {
            this.defaultStyleSheet = defaultStyleSheet;
        }
        public static int getCount()
        {
            return ChatSpeicher.count;
        }

        /**
         * 
         */
        public static void decrementCount()
        {
            count--;
        }

        /**
         * 
         */
        public static void incrementCount()
        {
            count++;
        }
        
        public long getDefaultExpirationTime()
        {
            return defaultExpirationTime;
        }
        
        public void setDefaultExpirationTime(long defaultExpirationTime)
        {
            this.defaultExpirationTime = defaultExpirationTime;
        }

        public void clearChatRoom(String chatRoomName)
        {
            myBuffers.clearChatRoom(chatRoomName);
        }

        // Private
        private void initLogger(String logfile)
        {
            try {
                Logger logger = Logger.getRootLogger();

                logger.removeAllAppenders();
                logger.setAdditivity(false);
                RollingFileAppender file_appender = new RollingFileAppender(
                        new PatternLayout("%5p (%F:%L) - %m%n"),logfile);
                file_appender.setMaxFileSize("10MB");
               
                logger.addAppender(file_appender);
            
            } catch (IOException e) {
                addMessage("Error appending LogFile.");
            }
        }
        private void setRootLogLevel(Level level) {
            Logger logger = Logger.getRootLogger();
            logger.setLevel(level);
            
            return;
        }
        
        private void removeMessages() {
            messages = new Vector();
        }
        private void addMessage(String message) {
            messages.addElement(message);
        }
        private void showMessages() {
            for(Enumeration e = messages.elements();e.hasMoreElements();) {
                logger.info((String) e.nextElement());
            }
            return;
        }
        
}

